@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static char id[] = "@(#)$Id: milter.c,v 8.50.4.44 2001/01/23 19:43:57 gshapiro Exp $";
+static char id[] = "@(#)$Id: milter.c,v 8.50.4.46 2001/05/11 18:11:36 gshapiro Exp $";
 #endif /* ! lint */
 
 #if _FFR_MILTER
@@ -62,7 +62,7 @@ static char *MilterEnvRcptMacros[MAXFILTERMACROS + 1];
 	    !isascii(response[2]) || !isdigit(response[2])) \
 	{ \
 		if (response != NULL) \
-			free(response); \
+			sm_free(response); \
 		response = newstr(default); \
 	} \
 	else \
@@ -74,7 +74,7 @@ static char *MilterEnvRcptMacros[MAXFILTERMACROS + 1];
 		{ \
 			if (*ptr == '%' && *++ptr != '%') \
 			{ \
-				free(response); \
+				sm_free(response); \
 				response = newstr(default); \
 				break; \
 			} \
@@ -354,7 +354,7 @@ milter_read(m, cmd, rlen, to, e)
 
 	if (milter_sysread(m, buf, expl, to, e) == NULL)
 	{
-		free(buf);
+		sm_free(buf);
 		return NULL;
 	}
 
@@ -1007,12 +1007,12 @@ milter_open(m, parseonly, e)
 			continue;
 		}
 		if (tTd(64, 5))
-			dprintf("X%s: error connecting to filter\n",
-				m->mf_name);
+			dprintf("X%s: error connecting to filter: %s\n",
+				m->mf_name, errstring(save_errno));
 		if (LogLevel > 0)
 			sm_syslog(LOG_ERR, e->e_id,
-				  "X%s: error connecting to filter",
-				  m->mf_name);
+				  "X%s: error connecting to filter: %s",
+				  m->mf_name, errstring(save_errno));
 		milter_error(m);
 # if _FFR_FREEHOSTENT && NETINET6
 		if (hp != NULL)
@@ -1681,7 +1681,7 @@ milter_send_macros(m, macros, cmd, e)
 	}
 	(void) milter_write(m, SMFIC_MACRO, buf, s,
 			    m->mf_timeout[SMFTO_WRITE], e);
-	free(buf);
+	sm_free(buf);
 }
 
 /*
@@ -1836,7 +1836,7 @@ milter_send_command(m, command, data, sz, e, state)
 	if (*state != SMFIR_REPLYCODE &&
 	    response != NULL)
 	{
-		free(response);
+		sm_free(response);
 		response = NULL;
 	}
 	return response;
@@ -1970,7 +1970,7 @@ milter_negotiate(m, e)
 				  "milter_negotiate(%s): returned %c instead of %c",
 				  m->mf_name, rcmd, SMFIC_OPTNEG);
 		if (response != NULL)
-			free(response);
+			sm_free(response);
 		milter_error(m);
 		return -1;
 	}
@@ -1986,7 +1986,7 @@ milter_negotiate(m, e)
 				  "milter_negotiate(%s): did not return valid info",
 				  m->mf_name);
 		if (response != NULL)
-			free(response);
+			sm_free(response);
 		milter_error(m);
 		return -1;
 	}
@@ -2005,7 +2005,7 @@ milter_negotiate(m, e)
 				  "milter_negotiate(%s): did not return enough info",
 				  m->mf_name);
 		if (response != NULL)
-			free(response);
+			sm_free(response);
 		milter_error(m);
 		return -1;
 	}
@@ -2014,7 +2014,7 @@ milter_negotiate(m, e)
 		      MILTER_LEN_BYTES);
 	(void) memcpy((char *) &pflags, response + (MILTER_LEN_BYTES * 2),
 		      MILTER_LEN_BYTES);
-	free(response);
+	sm_free(response);
 	response = NULL;
 
 	m->mf_fvers = ntohl(fvers);
@@ -2178,7 +2178,7 @@ milter_headers(m, e, state)
 		/* send it over */
 		response = milter_send_command(m, SMFIC_HEADER, buf,
 					       s, e, state);
-		free(buf);
+		sm_free(buf);
 		if (m->mf_state == SMFS_ERROR ||
 		    m->mf_state == SMFS_DONE ||
 		    *state != SMFIR_CONTINUE)
@@ -2280,7 +2280,7 @@ milter_body(m, e, state)
 			*state = SMFIR_TEMPFAIL;
 			if (response != NULL)
 			{
-				free(response);
+				sm_free(response);
 				response = NULL;
 			}
 		}
@@ -2522,7 +2522,7 @@ milter_changeheader(response, rlen, e)
 	if (h != sysheader && h->h_value != NULL)
 	{
 		e->e_msgsize -= strlen(h->h_value);
-		free(h->h_value);
+		sm_free(h->h_value);
 	}
 
 	if (*val == '\0')
@@ -2889,7 +2889,7 @@ milter_connect(hostname, addr, e, state)
 
 	response = milter_command(SMFIC_CONNECT, buf, s,
 				  MilterConnectMacros, e, state);
-	free(buf);
+	sm_free(buf);
 
 	/*
 	**  If this message connection is done for,
@@ -2916,7 +2916,7 @@ milter_connect(hostname, addr, e, state)
 			*state = SMFIR_REJECT;
 		if (response != NULL)
 		{
-			free(response);
+			sm_free(response);
 			response = NULL;
 		}
 	}
@@ -3042,7 +3042,7 @@ milter_envfrom(args, e, state)
 	/* send it over */
 	response = milter_command(SMFIC_MAIL, buf, s,
 				  MilterEnvFromMacros, e, state);
-	free(buf);
+	sm_free(buf);
 
 	/*
 	**  If filter rejects/discards a per message command,
@@ -3106,7 +3106,7 @@ milter_envrcpt(args, e, state)
 	/* send it over */
 	response = milter_command(SMFIC_RCPT, buf, s,
 				  MilterEnvRcptMacros, e, state);
-	free(buf);
+	sm_free(buf);
 	return response;
 }
 /*
@@ -3382,7 +3382,7 @@ milter_data(e, state)
 			if (rcmd != SMFIR_REPLYCODE &&
 			    response != NULL)
 			{
-				free(response);
+				sm_free(response);
 				response = NULL;
 			}
 
@@ -3414,7 +3414,7 @@ finishup:
 			*state = SMFIR_TEMPFAIL;
 			if (response != NULL)
 			{
-				free(response);
+				sm_free(response);
 				response = NULL;
 			}
 		}
@@ -3446,7 +3446,7 @@ finishup:
 			*state = SMFIR_TEMPFAIL;
 			if (response != NULL)
 			{
-				free(response);
+				sm_free(response);
 				response = NULL;
 			}
 		}
